/**
 *
 */
package org.glassfish;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.fail;

import org.apache.log4j.spi.LoggingEvent;
import org.junit.Test;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;

/**
 * @author David Matějček
 */
public class LogoutNPETest extends ArquillianTest {

  /**
   * Tests user's login and logout and fails because of NPE in server.log
   * Does not fail if we add beans.xml to war1.war - but it should not be needed,
   * because we don't use any CDI in war1.war.
   */
  @Test
  public void loginlogout() {
    // to be sure that we cannot reach this page without authorization.
    driver.navigate().to(url + "/war2/protected2.jsp");
    assertEquals("Unauthorized page title", "Unauthorized", driver.getTitle());

    driver.navigate().to(url + "/protected.jsp");
    assertEquals("Login screen title", "Login screen", driver.getTitle());
    WebElement login = driver.findElement(By.name("j_username"));
    login.click();
    login.sendKeys("tester");
    login = driver.findElement(By.name("j_password"));
    login.sendKeys("testPassword");
    driver.findElement(By.name("submit")).click();
    assertEquals("Protected welcome page title", "Protected welcome page", driver.getTitle());

    driver.navigate().to(url + "/war2/protected2.jsp");
    assertEquals("Another protected page title", "Another protected welcome page in another webapp", driver.getTitle());

    // WARN: this is not maintaible test - beanManager impl toString may return another string
    // If you add beans.xml to war1, the result will be true - this is not a bug, but helps
    // to understand the issue.
    driver.navigate().to(url + "/test");
    assertFalse("Response does not contain string beanManager", driver.getPageSource().contains("beanManager"));

    driver.navigate().to(url + "/logout.jsp");
    assertEquals("Logout screen title", driver.getTitle(), "Logout screen");

    driver.navigate().to(url + "/protected.jsp");
    assertEquals("Login screen title", driver.getTitle(), "Login screen");

    final LoggingEvent event = errorAppender.pop();
    if (event != null) {
      fail("Logging event observed, see logs: " + event.getMessage() + ": "
          + event.getThrowableInformation().getThrowable());
    }
  }
}
